//---------------------------------------------------------------------
// ATtiny26 8MHz
// Serial to NXT

// http://nicotak.com 
// by Takuya Matsubara / NICO Corp.2008-2009
//

// program internal 8MHz
//avrsp -FL11100100

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
#include "softi2cs.h"


void wait_msec(int i);
void sio_tx(char sio_txdat);
void timer_wait(unsigned char cnt);


#define MYSLA  0x02    // slave addressiUltrasonicj


#ifndef F_CPU
#define   F_CPU   8000000    //CPUNbNg 8MHz
#endif

#define BAUDRATE 9600        //{[[g

#define T0CS   8
#define T0CSCR 2
// 1=div1 / 2=div8 / 3=div64

#if BAUDRATE==2400
	#define T1CS  16        //vXP[l
	#define T1CSCR 5        //vXP[ݒl
#endif
#if BAUDRATE==4800
	#define T1CS  8        //vXP[l
	#define T1CSCR 4        //vXP[ݒl
#endif
#if BAUDRATE==9600
	#define T1CS  4        //vXP[l
	#define T1CSCR 3        //vXP[ݒl
#endif

#define SIO_MAXCNT  0x100 //^C}ől

#define SIO_T0BITCNT    ((F_CPU/T0CS)/BAUDRATE)
#define SIO_T0CNT       (SIO_MAXCNT-SIO_T0BITCNT)

#define SIO_T1BITCNT    ((F_CPU/T1CS)/BAUDRATE)
#define SIO_T1CNT       (SIO_MAXCNT-SIO_T1BITCNT)
#define SIO_T1CNTHALF   (SIO_MAXCNT-(SIO_T1BITCNT/2))

#define TCNT_100US  (SIO_MAXCNT-((F_CPU/T1CS)/(1000000/100)))
#define TCNT_80US  (SIO_MAXCNT-((F_CPU/T1CS)/(1000000/80)))
#define TCNT_5US  (SIO_MAXCNT-((F_CPU/T1CS)/(1000000/5)))
#define TCNT_2US  (SIO_MAXCNT-((F_CPU/T1CS)/(1000000/2)))
#define TCNT_1US  (SIO_MAXCNT-((F_CPU/T1CS)/(1000000/1)))

#define BUFFMAX 32

unsigned char sio_txwork;
char sio_txbitcnt = 0;

unsigned char sio_rxwork=0;
char sio_rxbitcnt = 0;

//ring buffer
unsigned char sio_buffer[BUFFMAX];
char sio_buffcnt=0;
char sio_readcnt=0;

//*********************
// Pin definition file
//*********************

#define LED_MASK	(1<<6)
#define LED_PORT   PORTB   // |[g
#define LED_DDR    DDRB    // o͐ݒ
#define LED_ON     LED_PORT|=LED_MASK
#define LED_OFF    LED_PORT&=~LED_MASK

#define SIORX_PIN   PINA
#define SIORX_PORT  PORTA
#define SIORX_DDR   DDRA
#define SIORX_BIT   (1<<7)
#define SIOTX_BIT   (1<<0)          //Mrbg

#define RXINTR_ENABLE	GIMSK|=(1<<PCIE1)	// 荞݋
#define RXINTR_DISABLE	GIMSK&=~(1<<PCIE1)	// 荞݋֎~

#define SIOTX_PORT PORTA      //TXp|[g
#define SIOTX_DDR  DDRA       //TXp|[go͐ݒ

//#define SIOTX_MARK	SIO_PORT&=~SIOTX_BIT	// ʐM
//#define SIOTX_SPACE	SIO_PORT|=SIOTX_BIT	// 0

#define SIOTX_MARK	SIOTX_PORT|=SIOTX_BIT	// ʐM
#define SIOTX_SPACE	SIOTX_PORT&=~SIOTX_BIT	// 0


//----------init
void sio_init(void)
{
	// VA|[g
	SIOTX_DDR |= SIOTX_BIT;
    SIOTX_MARK;	//ʐM

    TCCR0 = T0CSCR;       // ^C}0 vXP[ݒ

    TCCR1A = 0;            // ^C}1 [h 
    TCCR1B = T1CSCR;       // ^C}1 vXP[ݒ

	wait_msec(1000);	//1000msec

//init intr.
	PORTA = 0xff & ~SIORX_BIT;		//Pullup
	PORTB = 0xff & ~LED_MASK;		//Pullup

	ACSR |= (1<<ACD);	//Disable Analog Comparator 
	GIFR |= (1<<PCIF);	// clear Pin change flag 
 	RXINTR_ENABLE;	// RX荞݋

	sei();
}


//---------- interrupt
// RX荞
SIGNAL (SIG_PIN_CHANGE)
{
// Routine entered at falling edge
	if(SIORX_PIN & SIORX_BIT)
		return;	// オȂsv

	RXINTR_DISABLE;

    TCNT1 = SIO_T1CNTHALF;   // ^C}̏lݒ   
    TIFR |= (1 << TOV1);    // ^C}I[o[t[tONA
    TIMSK |= (1 << TOIE1);  // ^C}I[o[t[荞݋

	LED_ON;
	sio_rxbitcnt = 10;
}

//-----------------^C}荞(Mp)
SIGNAL (SIG_OVERFLOW0)
{
    TCNT0 = SIO_T0CNT;
//    TIFR |= (1 << TOV0);  // TIFR̃rbgNA

	sio_txbitcnt--;
	if(sio_txbitcnt <= 1){
  	    SIOTX_MARK;	// stopbit
		if(sio_txbitcnt <= 0){
			TIMSK &= ~(1 << TOIE0);  // ^C}I[o[t[荞disable
		}
		return;
	}

    //f[^rbg0-7(ʂ瑗M)
   	if(sio_txwork & 1)
	    SIOTX_MARK;		// 1
    else
   	   	SIOTX_SPACE;	// 0

	sio_txwork >>= 1;
}

//------------------^C}荞(Mp)
SIGNAL (SIG_OVERFLOW1)
{
    TCNT1 = SIO_T1CNT;   // ^C}̏lݒ   
//    TIFR |= (1 << TOV1);    // ^C}I[o[t[tONA
	sio_rxbitcnt--;
//	if(sio_rxbitcnt==9){	//start bit
//		return;
//	}
	if(sio_rxbitcnt==0){	//stop bit
		sio_buffer[(int)sio_buffcnt++] = sio_rxwork;

		if(sio_buffcnt >= BUFFMAX)
			sio_buffcnt=0;

		TIMSK &= ~(1 << TOIE1);  // ^C}I[o[t[荞disable
		LED_OFF;
		GIFR |= (1<<PCIF);	// clear Pinchange flag 
		RXINTR_ENABLE;
		return;
	}
	//bit8bitM
	sio_rxwork >>= 1;
	if(SIORX_PIN & SIORX_BIT)
    	sio_rxwork |= 0x80;
}

//--------Mobt@
unsigned char sio_isit_rx(void)
{
	if(sio_buffcnt == sio_readcnt) return(0);
	return(1);
}

//----------keyboard get buffer
unsigned char sio_getrxbuff(void)
{
	unsigned char temp = sio_buffer[(int)sio_readcnt++];

	if(sio_readcnt >= BUFFMAX)
		sio_readcnt = 0;

	return(temp);
}



//-------------------------------------------
// EGCg
void timer_wait(unsigned char cnt)
{
    TCNT1 = cnt;
    TIFR |= (1 << TOV1);  // TIFR̃rbgNA

    while(!(TIFR & (1 << TOV1)));
    // TOV1rbg1ɂȂ܂
}
//----------------------------
void wait_msec(int i)
{
	char z;

	while(i--){
		for(z=0; z<10; z++){
			timer_wait(TCNT_100US);
		}
	}
}


//-----------------------------------------------------------------------
// 1oCgM
// sio_txdatFMf[^0x00-0xff

void sio_tx(char sio_txdat)
{
	sio_txwork = sio_txdat;
	sio_txbitcnt = 10;

   	SIOTX_SPACE;	// 0

    TCNT0 = SIO_T0CNT;
    TIFR |= (1 << TOV0);  // TIFR̃rbgNA
	TIMSK |= (1 << TOIE0);  // ^C}I[o[t[荞disable
}

//---------------------------
int main(void)
{
	unsigned char slaw;
	unsigned char cmd;
	unsigned char cmd2;
	unsigned char ackdat;
	unsigned char val;

    i2c_init();

	LED_DDR |= LED_MASK;
//	LED_ON;

	sio_init();
	LED_OFF;

	//debug
//	while(1)
//	{
//		LED_PORT |= LED_MASK;	//LED on
//		sio_tx('A');
//		LED_PORT &= ~LED_MASK;	//LED off
//		wait_msec(1000);	//1000msec
//	}

	while(1)
	{
        while(1){
            i2c_start();               // }X^̐ڑ҂
            slaw = i2c_read_byte();    // AhXM
            if (slaw == MYSLA) {       // AhXv̏ꍇA
                i2c_write_bit(0,1);    // ACKM
                break;
            } else {                   // AhXsv̏ꍇA
                i2c_write_bit(1,1);    // NACKM
            }
        }
//		LED_ON;

        cmd = i2c_read_byte();         // R}hM
        i2c_write_bit(0,1);            // ACKM

        switch(cmd){
        case 0x41:
            cmd2 = i2c_read_byte();    // f[^Mî܂ܔjj
            i2c_write_bit(0,1);        // ACKM
            break;

        case 0x42:	//NXT1oCgM

            i2c_start();               // ʐMĊJ
            cmd2 = i2c_read_byte();    // AhXM
            i2c_write_bit(0,1);        // ACKM

			if(sio_isit_rx()){	//Mf[^
				val = sio_getrxbuff();	
			}else{
				val = 0;	//Mf[^Ȃ
			}

            i2c_write_byte(val);  // lM
            ackdat = i2c_read_bit(1);	//ack
            i2c_sda_low();
            break;
        case 0x43:	//NXT2oCgMioCiΉj
            i2c_start();               // ʐMĊJ
            cmd2 = i2c_read_byte();    // AhXM
            i2c_write_bit(0,1);        // ACKM

			if(sio_isit_rx()){	//Mf[^
    	        i2c_write_byte(1);  // M
	            ackdat = i2c_read_bit(1);	//ack
    	        i2c_write_byte(sio_getrxbuff());  // M

			}else{	//Mf[^Ȃ
    	        i2c_write_byte(0);  // M
    	        ackdat = i2c_read_bit(1);	//ack
    	        i2c_write_byte(0);  // M
			}

            ackdat = i2c_read_bit(1);	//ack
            i2c_sda_low();
            break;

        case 0x44:
            val = i2c_read_byte();    // f[^M
            i2c_write_bit(0,1);        // ACKM
			sio_tx(val);	//VAM
            break;

        default:
            break;
        }
        i2c_stop();
//		LED_OFF;
//		sio_tx(cmd);	//debug
//		sio_tx(val);	//debug
	}
	return(0);
}


